#!/usr/bin/perl -w
use strict;
use POSIX;

my $sigset = POSIX::SigSet ->new();
my $actionHUP=POSIX::SigAction->new("sigHUP_handler",$sigset,&POSIX::SA_NODEFER);
my $actionInt=POSIX::SigAction->new("sigINT_handler",$sigset,&POSIX::SA_NODEFER);
POSIX::sigaction(&POSIX::SIGHUP, $actionHUP);
POSIX::sigaction(&POSIX::SIGINT, $actionInt);
POSIX::sigaction(&POSIX::SIGTERM, $actionInt);

my $configFilePath ="/home/dgas/edg-wl-dgas-clientd.conf"; 
#my $configFilePath = $ARGV[0];

my %configValues = (
	dgasURDirs       => "/home/dgas/",
	qDepth 	         => "10",
	qMult	         => "3",
	lockFileName     => "/opt/edg/var/edg-wl-dgas-clientd.lock",
	mainPollInterval => "5",
	queuePollInterval =>"25",
	);

#Parse configuration file
if(exists $ARGV[0]) 
{
	$configFilePath = $ARGV[0];
}	
&parseConf($configFilePath); 

## Global variable initialization
my %hash;
my $UR;
my $list;
my @queue = ();

my $qDept = $configValues{qDepth};
my $qMult = $configValues{qMult}; 

my $some_dir      = $configValues{dgasURDirs};
my $lockFileName = $configValues{lockFileName};

my $T2 = $configValues{queuePollInterval};
my $T1 = $configValues{mainPollInterval};
my $keepGoing = 1;

my $start = time;   ##  time init

#put lock
if ( &putLock($lockFileName) != 0 )
{
	die "Fatal Error: Couldn't open lock file! in $lockFileName\n";
}
else
{
	print "Lock file succesfully created.\n";
}


while( $keepGoing )
{

##---------------------------------------------------------------- dir
    opendir(DIR, $some_dir) || die "can't opendir $some_dir: $!";
    while( my $file = readdir(DIR))
    {
	if( &matchFile($file) ==0 )           ## if file is not .* or .proxy
	{  
	    if(&parseFile($file)==0)          ## if file is a true input file   
	    {
		#  &printOut;
		if(&callAtmClient($file) == 0){&delFile($file);}
		else{&addToQueue($file);}
	    }   
	}
    }
    closedir DIR; 
    sleep($T1);
##----------------------------------------------------------------  
##----------------------------------------------------------------  queue 
    if(&timeout == 0)    ##timeout reached
    {
	if($#queue != -1)  ## if queue is not empty
	{
	    my $n=0;    ## count position of every file  in queue
	    my $max=$#queue+1;
	    
	    for(my $n=0; $n < $max; $n++)
	    {
		my @vec = split(/:/, $queue[$n]);
		&parseFile($vec[0]);
		
		if(&callAtmClient($vec[0]) == 0)
		{
		    splice @queue , $n, 1;
		    $max=$#queue+1; $n--;
		    &delFile($vec[0]);
		}
		else
		{
		    &setFile($vec[0], $vec[1], $vec[2], $n);
		}
	    }  	
	    
	    @queue = sort {($a =~ /:(\d*)$/)[0] <=> ($b =~ /:(\d*)$/)[0]} @queue;
	}
    }
##-----------------------------------------------------------------
}   

print "Exiting...\n";
if ( &delLock($lockFileName) != 0 )
{	
	print "Error removing lock file.\n";
}
else
{
	print "Lock file removed.\n";
}
print "Exit.\n";


###-----------------------END-----------------------------------------###

sub addToQueue {
   
    my $file= "$_[0]:0:0";
    my $num = $#queue;   
    
    if($num ==-1){$queue[0]= $file;}
    else{$queue[$num+1]= $file;}
    
    @queue = sort {($a =~ /:(\d*)$/)[0] <=> ($b =~ /:(\d*)$/)[0]} @queue;
}


sub callAtmClient {
    my $cmd = "";
    my $status = -1;
    $cmd = "/opt/edg/libexec/edg-wl-dgas-atmClient";
    $cmd .= " --jobid $hash{jobId}";
    $cmd .= " --resgridid $hash{server}:2119/jobmanager-pbs-$hash{queue}";
    $cmd .= " --remhlrid \"$hash{hlrLoc}\"";
    $cmd .= " CPU_TIME=$hash{cput} WALL_TIME=$hash{walltime}";
    `chown $hash{user}:root $some_dir/$_[0].proxy`; #
    $ENV{X509_USER_PROXY} = "$some_dir/$_[0].proxy"; 
    $status = system("su  $hash{user}  -c '$cmd'");
    print "$cmd\n";
    print "$_[0]\n";
    print "EXITSTATUS=$status\n";
    delete $ENV{X509_USER_PROXY};
    return $status;
}


sub delFile {   #receive as argument the filename to delete

    my $cmd = "rm -f $some_dir/$_[0]";
    print "$cmd \n";
    my $status = system($cmd);
    $cmd .=".proxy";
    print "$cmd \n";
    $status =system($cmd);
}


sub fakeAtm {

    print "insert atm status (0 or 1)\n";
    my $x;
    $x = <STDIN>;
    return $x;
}


sub matchFile {

    if(( $_[0] =~/^\..*/ ) || ($_[0] =~ ".proxy" )){return 1;}    ##  hidden file or a .proxy file   
    elsif($#queue == -1) {return 0;}
    else
    {
	foreach(@queue)
	{
	    my $file = $_;
	    if($file =~ /($_[0]:\d*:\d*)/){ return 1;}   ## file in queue
	}
	return 0;
    }	
}


sub parseFile {

    $list = "";
    $UR   = "";
    %hash = (
	     server   =>  "server",
	     queue    =>  "queue",
	     cput     =>  "cput",
	     hlrLoc   =>  "HLR_LOCATION",
	     jobId    =>  "EDG_WL_JOBID",
	     user     =>  "PBS_O_LOGNAME",
	     mem      =>  "mem",
	     vmem     =>  "vmem", 
	     walltime =>  "walltime",		   
	     );    
    
    open(FILE, "< $some_dir/$_[0]") || die "Open file $_ error!\n\n";
    my $line = readline(*FILE);	
    
    if($line !~ /^(Job Id:)/){return 1;}
    else
    {
	while(<FILE>)
	{
	    if(/server = (.*)$/){$hash{server}=$1;}
	    if(/queue = (.*)$/){$hash{queue}=$1;}
	    if(/Variable_List =/){$list=$list.$_;}
	    elsif(/^\t/){$list=$list.$_;}
	    if(/UR:(.*)$/){$UR=$1;}
	}
	
	close(FILE);
	&parseList($list);
	&parseUR($UR);
	return 0;
    }
}


sub parseList {

    $_[0]=~ s/[\n\t]//g;
    my @key = split (/,/ , $_[0]);
    
    foreach (@key)
    {
	if(/$hash{hlrLoc}=(.*)$/){$hash{hlrLoc}=$1;}
	if(/$hash{jobId}=(.*)$/){$hash{jobId}=$1;}
	if(/$hash{user}=(.*)$/){$hash{user}=$1;}
    }
}


sub parseUR {
    
    $_[0]=~ /cput=(\d\d):(\d\d):(\d\d).?mem=(\d*[M.k]b).?vmem=(\d*[M.k]b).?walltime=(\d\d):(\d\d):(\d\d)$/;  
    $hash{cput}     = $3 + $2*60 + $1*3600;
    $hash{walltime} = $8 + $7*60 + $6*3600;
    $hash{mem}      = $4;
    $hash{vmem}     = $5;
}       


sub printOut {

    print "queue: $hash{queue} \n";      
    print "server: $hash{server} \n";      
    print "HLR_LOCATION: $hash{hlrLoc} \n";      
    print "EDG_WL_JOBID: $hash{jobId} \n";      
    print "cput: $hash{cput} \n";
    print "mem: $hash{mem} \n";
    print "vmem: $hash{vmem} \n";
    print "walltime: $hash{walltime} \n";  
    print "------------------------------------ \n";
}


sub setFile {          ## 0: name, 1: mult, 2: prior, 3, pos in queue

    if($_[2]>=$qDept){return 1;}
    else
    {
	$_[1]++;
	if($_[1] >= $qMult)
	{
	    $_[2]++;
	    $_[1]=0;
	}
	my $file = "$_[0]:$_[1]:$_[2]";
	$queue[$_[3]] = $file;
	return 0;
    }
}


sub timeout {
    
    my $stop = time;
    my $delta = $stop -$start;
    print "delta = $delta \n" ;   
    
    if($delta > $T2)
    {
	$start= time;
	return 0;
    }
    else {return 1;}
}

sub putLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") && return 1;
    close(IN);

    open(OUT, "> $lockName") || return 2;
    print OUT  $$;    ## writes pid   
    close(OUT);
    return 0;
}

sub delLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") || return 1;
    close(IN);
    my $status = system("rm -f $lockName");
    return $status;
}

sub sigHUP_handler {
        print "got SIGHUP\n";
	$keepGoing = 1;
}

sub sigINT_handler {
        print "got SIGINT\n";
        $keepGoing = 0;
}


sub parseConf {
    my $fconf = $_[0];
    open(FILE, "$fconf") || die "file not open \n";
    while(<FILE>)
    {
	if(/^qDept\s*=\s*\"(.*)\"$/){$configValues{qDept}=$1;}
	if(/^qMult\s*=\s*\"(.*)\"$/){$configValues{qMult}=$1;} 
	if(/^dgasURDirs\s*=\s*\"(.*)\"$/){$configValues{dgasURDirs}=$1;}
	if(/^lockFileName\s*=\s*\"(.*)\"$/){$configValues{lockFileName}=$1;}
	if(/^mainPollInterval\s*=\s*\"(.*)\"$/){$configValues{mainPollInterval}=$1;}
	if(/^queuePollInterval\s*=\s*\"(.*)\"$/){$configValues{queuePollInterval}=$1;}
    }
    close(FILE);
}
